"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.performanceTracerFactory = void 0;

var _awsXraySdkCore = _interopRequireDefault(require("aws-xray-sdk-core"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const performanceTracerFactory = () => {
  let segment = process.env.TRACE ? _awsXraySdkCore.default.getSegment() : null;
  let traceId = process.env._X_AMZN_TRACE_ID;
  return {
    getSegment: () => {
      if (traceId !== process.env._X_AMZN_TRACE_ID) {
        traceId = process.env._X_AMZN_TRACE_ID;
        segment = process.env.TRACE ? _awsXraySdkCore.default.getSegment() : null;
      }

      return {
        addNewSubsegment: subsegmentName => {
          const subsegment = process.env.TRACE ? segment.addNewSubsegment(subsegmentName) : null;
          const prevSegment = segment;
          segment = subsegment;
          return {
            addAnnotation: (annotationName, data) => {
              if (process.env.TRACE) {
                void subsegment.addAnnotation(annotationName, data != null ? data : '<Empty annotation>');
              }
            },
            addError: error => {
              if (process.env.TRACE) {
                void subsegment.addError(error);
              }
            },
            close: () => {
              if (process.env.TRACE) {
                void subsegment.close();
                segment = prevSegment;
              }
            }
          };
        }
      };
    }
  };
};

exports.performanceTracerFactory = performanceTracerFactory;
//# sourceMappingURL=performance-tracer-factory.js.map