"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sendReactiveEvent = void 0;

var _runtimeBase = require("@resolve-js/runtime-base");

var _postgres = require("resolve-cloud-common/postgres");

var _lambda = require("resolve-cloud-common/lambda");

var _utils = require("resolve-cloud-common/utils");

const sendReactiveEvent = async event => {
  const {
    aggregateId,
    type
  } = event;
  const log = (0, _runtimeBase.getLog)(`sendReactiveEvent:${event.type}`);
  const databaseNameAsId = (0, _postgres.escapeId)(process.env.RESOLVE_EVENT_STORE_DATABASE_NAME);
  const subscriptionsTableNameAsId = (0, _postgres.escapeId)(process.env.RESOLVE_SUBSCRIPTIONS_TABLE_NAME);
  const connectionIdsResult = await (0, _postgres.executeStatement)({
    Region: process.env.AWS_REGION,
    ResourceArn: process.env.RESOLVE_EVENT_STORE_CLUSTER_ARN,
    SecretArn: process.env.RESOLVE_USER_SECRET_ARN,
    Sql: `SELECT "connectionId" FROM ${databaseNameAsId}.${subscriptionsTableNameAsId}
          WHERE (
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."eventTypes" #>
             ${(0, _postgres.escapeStr)(`{${JSON.stringify(type)}}`)} =
             CAST('true' AS jsonb)
            OR
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."eventTypes" #> '{}' =
             CAST('null' AS jsonb) 
            ) AND (
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."aggregateIds" #>
             ${(0, _postgres.escapeStr)(`{${JSON.stringify(aggregateId)}}`)} =
             CAST('true' AS jsonb)
            OR
             ${databaseNameAsId}.${subscriptionsTableNameAsId}."aggregateIds" #> '{}' =
             CAST('null' AS jsonb) 
          )`
  });
  const errors = [];
  await Promise.all(connectionIdsResult.map(({
    connectionId
  }) => (0, _lambda.invokeFunction)({
    Region: process.env.AWS_REGION,
    FunctionName: process.env.RESOLVE_WEBSOCKET_LAMBDA_ARN,
    Payload: {
      type: 'send',
      connectionId,
      data: {
        type: 'events',
        payload: {
          events: [event]
        }
      }
    }
  }).catch((0, _utils.errorBoundary)(errors))));

  if (errors.length > 0) {
    log.warn(`Failed push event to websocket. ${errors}`);
  }
};

exports.sendReactiveEvent = sendReactiveEvent;
//# sourceMappingURL=send-reactive-event.js.map